local RL                = AceLibrary("Roster-2.1")
local L                 = AceLibrary("AceLocale-2.2"):new("eeMon")

local NameWidth = 90
local DmgWidth = 50
local Padding = 4

local testing = false;
local testDuration = 0;


--[[
name:    BuildDisplay()
desc:    Create all the GUI elements we won't change (or won't change in our
         update functions) when our addon is enabled.  We'll then need to call 
         our UpdateXXX() methods for the first time, to make sure the gui is set
]]--
function eeMon:BuildDisplay()

	self.frames = self.frames or {}
	self.frames.units = self.frames.units or {}

	-- Create our main frame to hold all our gui elements
	local f = CreateFrame("Frame", "MainFrame", UIParent)
	f:SetMovable(true)
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.x, self.db.profile.y)
	f:SetClampedToScreen(true)
	-- hopefully this will stop the frame from capturing mouse clicks
	f:EnableMouse(false)
	self.frames.main = f

	-- Create a button to hold our title
	-- We'll use a button so we can move our frame by clicking on it
	local title = CreateFrame("Button", "Title", self.frames.main)
	title:EnableMouse(true)                            
	title:SetHeight(16)
	title:SetPoint("TOP", self.frames.main, "TOP", 0, 0)
	title:SetScript("OnMouseUp", self.StopMoveFrame)
	title:SetScript("OnMouseDown",self.StartMoveFrame)
	self.frames.title = title

	-- Create the text for our title button
	local text = title:CreateFontString(nil, "ARTWORK")
	text:SetFontObject(GameFontHighlightSmall)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("BOTTOM")
	text:SetHeight(12)
	text:SetPoint("TOP", self.frames.title, "TOP", 0, -5)
	self.frames.titletext = text
   
	local bg = CreateFrame("Frame", "Background", self.frames.main)
	bg:SetFrameLevel(0)
	bg:SetPoint("TOP",self.frames.title,"BOTTOM",0,0)
	bg:Show()
	self.frames.bg = bg

	local texture = self.frames.bg:CreateTexture(nil, "LOW")
	texture:SetPoint("CENTER", self.frames.bg, "CENTER",0,0)
	self.frames.bg.texture = texture


	-- Finish setting up our frames/bars
	self:UpdateFrame()
	self:UpdateBars()

	-- Set the scale for the entire GUI.  Make sure to do this last
	self.frames.main:SetScale(self.db.profile.DisplayScale)
end



--[[
name:    UpdateFrame()
desc:    Rebuilds all frame elements based on user inputs
]]--
function eeMon:UpdateFrame()
   
	-- Adjust frame size if name is placed inside the bar
	local currNameWidth = NameWidth
	if (eeMon.db.profile.NamePosition == L["InsideBar"]) then
		currNameWidth = 0
	end

	-- Adjust frame size if missing health is hidden or placed inside the bar
	local currDmgWidth = 0;
	if (eeMon.db.profile.HealthPosition == L["OutsideBar"]) then
		currDmgWidth = DmgWidth + Padding
	end


	-- Sum width and height
	local totalWidth = self.db.profile.BarWidth + currNameWidth + currDmgWidth + (Padding * 3) -- padding for frame to name, name to bar, bar to health/frame
	local totalHeight = self.db.profile.ScanUnits * (self.db.profile.BarHeight + self.db.profile.BarSpacing) + self.db.profile.BarSpacing + 5 + 5

	-- Main frame
	self.frames.main:SetWidth(totalWidth)
	self.frames.main:SetHeight(totalHeight)


	-- Frame background
	self.frames.bg:SetAlpha(self.db.profile.FrameAlpha)
	self.frames.bg:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, 
		tileSize = 32, edgeFile=self:GetBorder(), edgeSize = 16,
		insets = {left = 5, right = 5, top = 5, bottom = 5},
	})
	self.frames.bg:SetWidth(self.frames.main:GetWidth())
	self.frames.bg:SetHeight(self.frames.main:GetHeight())

	-- Frame background color
	self.frames.bg.texture:SetTexture(self.db.profile.FrameColor.r, 
	self.db.profile.FrameColor.g, self.db.profile.FrameColor.b)
	self.frames.bg.texture:SetWidth(self.frames.bg:GetWidth()-10)
	self.frames.bg.texture:SetHeight(self.frames.bg:GetHeight()-10)

	-- Frame title
	self.frames.title:SetWidth(totalWidth)

	-- Frame title text
	self.frames.titletext:SetWidth(totalWidth)
	self.frames.titletext:SetText(self:GetMyTitle())
end



--[[
name:    UpdateBars()
desc:    Rebuilds all the bars elements based on user inputs.
]]--
function eeMon:UpdateBars()
 	
   -- Hide all frame units
   for i in pairs(self.frames.units) do self.frames.units[i]:Hide() end
   
	for i = 1, self.db.profile.ScanUnits do
		local frame
		local name
		local bar
		local bagBG
		local dmg
		
		-- If we haven't created a frame for this unit position before, make it
		if self.frames.units[i] == nil then
			-- Create the gui items
			frame = CreateFrame("Button", nil, self.frames.main)
			name  = frame:CreateFontString(nil, "ARTWORK")
			barBG = CreateFrame("StatusBar", nil, frame)
			bar   = CreateFrame("StatusBar", nil, frame)
			dmg   = frame:CreateFontString(nil,"ARTWORK")
			
			-- Set up (mostly) static variables
			frame:Hide()
			frame:EnableMouse(true)
			frame:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp", "Button4Up", "Button5Up")
			frame:SetParent(self.frames.main)
			
			name:SetFontObject(GameFontHighlightSmall)
			name:SetHeight(12)
			
			dmg:SetFontObject(GameFontHighlightSmall)
			dmg:SetHeight(12)
			
			barBG:SetMinMaxValues(0,100)
			barBG:SetFrameLevel(1)
			
			bar:SetMinMaxValues(0,100)
			bar:SetFrameLevel(2)
			
			-- Add it to our frames to display
			frame.Name = name
			frame.Bar = bar
			frame.BarBG = barBG
			frame.Dmg = dmg
			self.frames.units[i] = frame
			
		-- If we have, just get references to the items
		else
			frame = self.frames.units[i]
			name  = self.frames.units[i].Name
			barBG = self.frames.units[i].BarBG
			bar   = self.frames.units[i].Bar
			dmg   = self.frames.units[i].Dmg
		end
		
		
		
		-- See if user wants us to hide the missing health part
		if (eeMon.db.profile.HealthPosition == L["Hidden"]) then
			dmg:Hide()
		else
			dmg:Show()
		end
		
		-- We have to make sure we set the bar width to the total size, so every
		-- part of the bar can capture mouse clicks
		local currNameWidth = NameWidth
		if (eeMon.db.profile.NamePosition == L["InsideBar"]) then
			currNameWidth = 0
		end
		local currDmgWidth = 0;
		if (eeMon.db.profile.HealthPosition == L["OutsideBar"]) then
			currDmgWidth = DmgWidth + Padding
		end
		
		frame:SetWidth(self.db.profile.BarWidth + currNameWidth + currDmgWidth + Padding*2) 
		frame:SetHeight(self.db.profile.BarHeight)
		
		name:SetWidth(NameWidth)
		name:ClearAllPoints()
		
		dmg:SetWidth(DmgWidth + Padding)
		dmg:ClearAllPoints()
		-- If we're putting missing health in bar, use the right side of the text as an anchor
		if (eeMon.db.profile.HealthPosition == L["InsideBar"]) then
			dmg:SetJustifyH("RIGHT")
			dmg:SetPoint("RIGHT", frame.BarBG, "RIGHT", -Padding/2, 0)
		-- Else, use the left side of the text as an anchor
		else
			dmg:SetJustifyH("LEFT")
			dmg:SetPoint("LEFT", frame.BarBG, "RIGHT", Padding, 0)
		end
		
		
		barBG:SetStatusBarTexture( "Interface\\AddOns\\eeMon\\images\\" .. eeMon.db.profile.BarTexture .. ".tga" )
		barBG:ClearAllPoints()
		barBG:SetWidth(self.db.profile.BarWidth)
		barBG:SetHeight(self.db.profile.BarHeight)
		
		
		bar:SetStatusBarTexture( "Interface\\AddOns\\eeMon\\images\\" .. eeMon.db.profile.BarTexture .. ".tga" )
		bar:ClearAllPoints()
		bar:SetWidth(self.db.profile.BarWidth)
		bar:SetHeight(self.db.profile.BarHeight)
		
		
		-- If we're putting the name in the bar, anchor to the left of the frame
		if (eeMon.db.profile.NamePosition == L["InsideBar"]) then
			name:SetPoint("LEFT", frame, "LEFT", Padding+Padding/2, 0)
			name:SetJustifyH("LEFT")
			barBG:SetPoint("LEFT", frame, "LEFT", Padding+Padding/2, 0)
			bar:SetPoint("LEFT", frame, "LEFT", Padding+Padding/2, 0)
		-- Else, anchor to the right of the name text
		else
			name:SetPoint("LEFT", frame, "LEFT", Padding-Padding/2, 0)
			name:SetJustifyH("RIGHT")
			barBG:SetPoint("LEFT", frame.Name, "RIGHT", Padding, 0)
			bar:SetPoint("LEFT", frame.Name, "RIGHT", Padding, 0)
		end
		
		
		-- Set each bar's position below the previously created one
		if i == 1 then
			frame:SetPoint("TOPLEFT", self.frames.bg, "TOPLEFT", 0, -5-self.db.profile.BarSpacing)
		else
			self.frames.units[i]:SetPoint("TOPLEFT", self.frames.units[i-1], "BOTTOMLEFT", 0, -(self.db.profile.BarSpacing))
		end
	
	end
      
end



--[[
name:    StartMoveFrame()
desc:    Used by our frame's title button, to signify that we're moving if the
         frame isn't locked, and the user clicked their left mouse button
]]--
function eeMon:StartMoveFrame()
   if not eeMon.db.profile.Lock and arg1 == "LeftButton" then
      eeMon.frames.main:StartMoving()
      eeMon.frames.main.isMoving = true
   end
end



--[[
name:    StopMoveFrame()
desc:    Used by our frame's title button, to save the frames position when
         it's been moved
]]--
function eeMon:StopMoveFrame()
   if eeMon.frames.main.isMoving then
      eeMon.frames.main:StopMovingOrSizing()
      eeMon.db.profile.x = eeMon.frames.main:GetLeft()
      eeMon.db.profile.y = eeMon.frames.main:GetTop()
      eeMon.frames.main.isMoving = false
   end
end



--[[
name:    Scale()
desc:    Scales the main gui frame (and thus, all this child elements), and
         re-calculates it's screen position based on it's scale
]]--
function eeMon:Scale()
	-- Get the current frame locs
	local x,y = self.db.profile.x,self.db.profile.y
	-- Multiply the locs by the current frame scale
	local s = self.frames.main:GetEffectiveScale()
	x,y = x*s,y*s
	-- Scale the frame
	self.frames.main:SetScale(self.db.profile.DisplayScale)
	-- Divide the locs by the new frame scale to get where we should be
	local s = self.frames.main:GetEffectiveScale()
	x,y = x/s,y/s
	-- Reset the frames position
	self.frames.main:ClearAllPoints()
	self.frames.main:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
	-- Save the location
	self.db.profile.x = x
	self.db.profile.y = y
end



--[[
name:    GetBorder()
desc:    We can just call this to display our border, or no border, based on
         user input
]]--
function eeMon:GetBorder()
   if self.db.profile.FrameBorderOn then
      return "Interface\\Tooltips\\UI-Tooltip-Border"
   else
      return nil
   end
end



--[[
name:    GetMyTitle()
desc:    We can just call this to display our title, or no title, based on
         user input
]]--
function eeMon:GetMyTitle()
   if self.db.profile.FrameTitleOn then
      return L["eeMon"]
   else
      return ""
   end
end



--[[
name:    CheckVisibility()
desc:    Check to display the GUI based on current group and user setting
]]--
function eeMon:CheckVisibility()
   -- Always display
   if self.db.profile.DisplayWhen == L["Always"] then
      self:ShowGUI()
   -- We're in a party; show
   elseif self.db.profile.DisplayWhen == L["Party"] and GetNumPartyMembers() > 0 then
      self:ShowGUI()
   -- We're in a raid; show
   elseif self.db.profile.DisplayWhen == L["Raid"] and GetNumRaidMembers() > 0 then
      self:ShowGUI()
   -- We're either set to not-show, or we're not in a prereq party/raid
   else
      self:HideGUI()
   end
end



--[[
name:    HideGUI()
desc:    Displays the GUI, and sets a timer to update the GUI w/ damaged units
]]--
function eeMon:ShowGUI()
   self.frames.main:Show()
   -- Register repeating AceEvent
   self:ScheduleRepeatingEvent("CheckUnitsID", self.CheckUnits, self.db.profile.ScanFrequency, self)
end



--[[
name:    HideGUI()
desc:    Hides the GUI, and cancels our update event so we don't waste cycles
]]--
function eeMon:HideGUI()
   self.frames.main:Hide()
   -- Unegister AceEvent
   self:CancelScheduledEvent("CheckUnitsID")
end



--[[
name:    UpdateFrequency()
desc:    Changes the scan/update frequency when someone modifies the value
]]--
function eeMon:UpdateFrequency()
	-- If we're in test mode, update the test speed.  The test will update the
	-- real scan frequency when its finished
	if testing then
		self:CancelScheduledEvent("CheckUnitsID")
		self:ScheduleRepeatingEvent("TestEventID", self.TestUpdate, self.db.profile.ScanFrequency, self)
	-- Otherwise, update the scan frequency like normal
	else
		self:CancelScheduledEvent("CheckUnitsID")
		self:ScheduleRepeatingEvent("CheckUnitsID", self.CheckUnits, self.db.profile.ScanFrequency, self)
	end
end


--[[
name:    GetNameColor()
desc:    Returns class color or white, based on users settings
]]--
function eeMon:GetNameColor(class)
   -- If we're usingn class colors, and we found the class in wow's 
   -- class color table...
	--if self.db.profile.ColorClass and RAID_CLASS_COLORS[class] then
   if self.db.profile.DisplayClassColor and RAID_CLASS_COLORS[class] then
      -- return the color
		local color = RAID_CLASS_COLORS[class]
		return color.r, color.g, color.b
   -- Otherwise, just return an off-white color
	else
		return 1, 1, 1
	end
end



--[[
name:    FindBarColor()
desc:    Returns the color and alpha for the bar based on user settings
]]--
function eeMon:FindBarColor(isBackground,unit)

   -- Default style: green health bar, black background
   --if unit == nil or eeMon.db.profile.BarColor == "Default" then
   if unit == nil or eeMon.db.profile.BarStyle == L["Default"] then
      if isBackground then
         return 0.4,0,0,eeMon.db.profile.BarAlpha
      else
         return 0,1,0,eeMon.db.profile.BarAlpha
      end
   -- Class style: bars colored by units class with a slightly darker background
   --elseif eeMon.db.profile.BarColor == "Class" then
   elseif eeMon.db.profile.BarStyle == L["Class"] then
      local color = RAID_CLASS_COLORS[unit.class]
      if isBackground then
         return color.r-0.4, color.g-0.4, color.b-0.4, eeMon.db.profile.BarAlpha
      else
         return color.r, color.g, color.b, eeMon.db.profile.BarAlpha
      end
   -- CTRaid style: red bar = player, blue = party member, green = everyone else
   else
      local color
      if UnitIsUnit(unit.unitid,"Player") then color = {r=1,g=0,b=0}
      elseif UnitInParty(unit.unitid) then color = {r=0,g=1,b=1}
      else color = {r=0,g=1,b=0}
      end
      
      if isBackground then
         return color.r-0.6, color.g-0.6, color.b-0.6, eeMon.db.profile.BarAlpha
      else
         return color.r, color.g, color.b, eeMon.db.profile.BarAlpha
      end
   end
   
end



--[[
name:		Test()
desc:		Pauses normal operation, and displays test bars using our settings
]]--
function eeMon:Test()

	-- Cancel test
	if testing then
		self:TestCleanup()
		
	-- Start test
	else
		-- Set our global so we know
		testing = true
		-- Show if we're hidden
		self:ShowGUI()
		-- Make sure normal scanning is disabled
		self:CancelScheduledEvent("CheckUnitsID")
		-- Schedule test event
		self:ScheduleRepeatingEvent("TestEventID", self.TestUpdate, 
      	self.db.profile.ScanFrequency, self)
	end
	
end



--[[
name:		TestUpdate()
desc:		Simple function which will generate display bars for a set amount of time
--]]
function eeMon:TestUpdate()

	-- Make sure the test ends after a certain amount of time
	if testDuration < 100 then
	
		-- Populate display with dummy data
		testUnit = RL:GetUnitObjectFromUnit("player")
		for i=1,eeMon.db.profile.ScanUnits do
			--eeMon.frames.units[i].Name:SetText( UnitName("player") )
			eeMon.frames.units[i].unit = testUnit
			eeMon.frames.units[i].Name:SetText(testUnit.name)
			eeMon.frames.units[i].Name:SetTextColor(self:GetNameColor(testUnit.class))
			eeMon.frames.units[i].BarBG:SetStatusBarColor( eeMon:FindBarColor(true,testUnit) )
			eeMon.frames.units[i].Bar:SetStatusBarColor( eeMon:FindBarColor(false,testUnit) )
			eeMon.frames.units[i].Bar:SetValue(100-testDuration)
			eeMon.frames.units[i].Dmg:SetText(" -" .. UnitHealthMax(testUnit.unitid) - UnitHealth(testUnit.unitid))
			eeMon.frames.units[i]:Show()
		end
			
		testDuration = testDuration + 1
	-- Call cleanup
	else
		self:TestCleanup()
	end
end



--[[
name:		TestCleanup()
desc:		Resets normal operation once a test is finished / canceled
]]--
function eeMon:TestCleanup()
	-- Reset globals
	testing = false
	testDuration = 0;
	-- Cancel test event
	self:CancelScheduledEvent("TestEventID")
	-- Go back to normal
	self:CheckVisibility()
end

