local proximity         = ProximityLib:GetInstance("1")
local RL                = AceLibrary("Roster-2.1")
local BC                = AceLibrary("Babble-Class-2.2")



--[[
name:    RosterLib_RosterUpdated()
desc:    If the roster has changed, we check to see if we should show/hide the
         gui, based on the users settings
]]--
function eeMon:RosterLib_RosterUpdated(rosterTable)
   eeMon:CheckVisibility()
end



--[[
name:    Filter()
desc:    Checks to see if we should display a unit based on our filter settings
]]--
function eeMon:Filter(unit)
   if not UnitIsUnit("Player",unit.unitid) and 
   (eeMon.db.profile.ClassFilter[unit.class] or
   (GetNumRaidMembers() > 0 and eeMon.db.profile.GroupFilter[unit.subgroup]))
   then
      return true
   elseif UnitIsUnit("Player",unit.unitid) and eeMon.db.profile.SelfFilter then
      return true
   else
      return false
   end
end



--[[
name:    InRange()
desc:    Returns if a unit is within the user specified scan range
]]--
function eeMon:InRange(unit)
   local range,time = proximity:GetUnitRange(unit.unitid)
   local seen = GetTime() - (time or 100)
   
   -- Unit might not be around to get its range, so we also find when it was seen
   if range ~= nil and range < eeMon.db.profile.ScanRange and seen < 3 then
      return true
   else
      return false
   end
end



--[[
name:    CheckUnits()
desc:    Checks all raid/party members and pets, performs our filter checks,
         and then displays the most damaged units based on user settings
]]--
function eeMon:CheckUnits()
      
   -- Compost will keep us from GC the table between calls
   myTable = {}
   
   -- RL:Iterate() seems to be buggy, so we'll do it ourselves
   for _, u in pairs(RL.roster) do
   
      -- Check if we should filter the unit out
      if not eeMon:Filter(u) and eeMon:InRange(u) then
      
         -- Get the current health percentage of this unit
         local pct = (UnitHealth(u.unitid) / UnitHealthMax(u.unitid)) * 100
         
         -- If the unit is damaged, we'll attempt to place it in our list
         if pct <= eeMon.db.profile.ScanHealth and not UnitIsDeadOrGhost(u.unitid) then
            self:CheckToShow(myTable,u,1)
         end
      end
      
   end
   
   -- Reset our display frames to show the current list of damaged units
   self:DisplayUnits(myTable)
end



--[[
name:    CheckToShow()
desc:    Checks if a unit is damaged enough to be displayed using recursive
         logic to check it against units already in the table.  Attempts to 
         re-sort units that have been bumped out of their spot
]]--
function eeMon:CheckToShow(tbl,unit,i)
   -- We shouldn't ever be below 1, but check for it.  We will be above the max
   -- when we've pushed a previous value out, and they no longer fit in the list
   if i > eeMon.db.profile.ScanUnits or i < 1 then return end

   -- Get health percent for the current unit
   local currPct  = UnitHealth(unit.unitid) / UnitHealthMax(unit.unitid)
   
   -- If we're checking a fresh unit, make sure that they belong in our list
   -- before checking for the position to place it
   if i == 1 and not tbl[eeMon.db.profile.ScanUnits] == nil then
      local checkPct = UnitHealth(tbl[eeMon.db.profile.ScanUnits].unitid) /
         UnitHealthMax(tbl[eeMon.db.profile.ScanUnits].unitid)
      if currPct > checkPct then return end
   end
   
   -- If there's nothing at this table entry, place the unit here and finish
   if tbl[i] == nil then
      tbl[i] = unit
      return
   end
   
   -- Otherwise, check our units health against the unit at the ith position
   -- in our table
   local checkPct = UnitHealth(tbl[i].unitid) / UnitHealthMax(tbl[i].unitid)

   -- If we're not as damaged as this unit, check against the next
   if currPct >= checkPct then
      self:CheckToShow(tbl,unit,i+1)
      
   -- We're more damage than this unit.  Replace it, see if there's still a
   -- place for this unit somewhere else in our table
   else
      local prevUnit = tbl[i]
      tbl[i] = unit
      self:CheckToShow(tbl,prevUnit,i+1)
   end
end



--[[
name:    DisplayUnits
desc:    Takes a table of damaged units and sets our GUI to display them
]]--
function eeMon:DisplayUnits(tbl)
   
   local pct = nil
   -- Loop through all the unit frame bars we've created since being enabled
   -- (we won't have more damaged units then there are bars)
	for i in pairs(eeMon.frames.units) do
		
      -- If there's a damaged unit to display for this frame, simply update that
      -- frames data, make sure it's shown.
      if tbl[i] then
			eeMon.frames.units[i].unit = tbl[i]
         pct = (UnitHealth(tbl[i].unitid) / UnitHealthMax(tbl[i].unitid)) * 100
         
         eeMon.frames.units[i].Name:SetText(tbl[i].name)
         eeMon.frames.units[i].Name:SetTextColor(self:GetNameColor(tbl[i].class))
         eeMon.frames.units[i].BarBG:SetStatusBarColor( eeMon:FindBarColor(true,tbl[i]) )
         eeMon.frames.units[i].Bar:SetStatusBarColor( eeMon:FindBarColor(false,tbl[i]) )
         eeMon.frames.units[i].Bar:SetValue(pct)
         eeMon.frames.units[i].Dmg:SetText(" -" .. UnitHealthMax(tbl[i].unitid) - UnitHealth(tbl[i].unitid))
         eeMon.frames.units[i]:Show()
         
      -- Otherwise, make sure any extra bars are hidden.  I deliberately don't
      -- hide all the frame bars, and then only show the necessary ones, as the
      -- frame bar will seem unresponsive to mouse clicks (the click is actually
      -- occurring between the hide/show) despite the bar always being visible
      else
         eeMon.frames.units[i]:Hide()
      end
      
   end
   
end
